@@ -440,6 +440,8 @@ ndraws.rvar <- function(x) {
 
 # check validity of existing variable names: e.g., that
 # all `variables` exist in `x` and that no `variables`are reserved words
+# Additionally, this returns the cannonical name, so e.g. "theta" will get
+# converted to c("theta[1]", "theta[2]", ...) if those variables exist.
 # @param regex should 'variables' be treated as regular expressions?
 # @param scalar_only should only scalar variables be matched?
 check_existing_variables <- function(variables, x, regex = FALSE,
@@ -461,14 +463,31 @@ check_existing_variables <- function(variables, x, regex = FALSE,
     missing_variables <- NULL
     variables <- as.character(all_variables[unique(unlist(tmp))])
   } else if (!scalar_only) {
-    missing_candidates <- setdiff(variables, all_variables)
-    all_variables_base <- gsub("\\[.*\\]$", "", all_variables)
-    missing_variables <- setdiff(missing_candidates, all_variables_base)
-    vector_variables <- setdiff(missing_candidates, missing_variables)
-    variables <- unique(
-      c(variables[!(variables %in% missing_candidates)],
-        all_variables[all_variables_base %in% vector_variables])
-    )
+    # need to find variables that are matched by either a scalar or vector
+    # variable in x and what the matching variable is, while keeping original
+    # order of input `variables`
+
+    # find scalar variables (1-to-1 match between all_variables and variables)
+    scalar_input_ixs <- match(all_variables, variables)
+    # find vector variable matches (match all_variables with the indexing stripped)
+    all_variables_base <- all_variables
+    # exclude already matched scalar variables
+    all_variables_base[!is.na(scalar_input_ixs)] <- NA_character_
+    all_variables_base <- gsub("\\[.*\\]$", "", all_variables_base, perl = TRUE)
+    vector_input_ixs <- match(all_variables_base, variables)
+    # compose the vector of indices of matched input variables
+    input_ixs <- c(scalar_input_ixs[!is.na(scalar_input_ixs)],
+                   vector_input_ixs[!is.na(vector_input_ixs)])
+    # compose the vector of indices of matched all_variables
+    all_var_ixs <- seq_along(all_variables)
+    all_var_matched_ixs <- c(all_var_ixs[!is.na(scalar_input_ixs)],
+                             all_var_ixs[!is.na(vector_input_ixs)])
+    # select missed input variables
+    missing_vars_mask <- rep_len(TRUE, length(variables))
+    missing_vars_mask[input_ixs] <- FALSE
+    missing_variables <- variables[missing_vars_mask]
+    # select matched all_variables maintaining the input variables order
+    variables <- all_variables[all_var_matched_ixs[order(input_ixs, all_var_matched_ixs)]]
   } else {
     missing_variables <- setdiff(variables, all_variables)
   }
