@@ -50,6 +50,13 @@ test_that("rvar constructor using with_chains works", {
   expect_error(rvar(1, with_chains = TRUE))
 })
 
+test_that("NULL rvar creates a 0-length numeric rvar with 1 draw", {
+  expect_equal(rvar(), rvar(numeric()))
+  expect_equal(rvar(NULL), rvar(numeric()))
+  expect_equal(as_rvar(NULL), rvar(numeric()))
+  expect_equal(draws_of(rvar()), array(numeric(), dim = c(1, 0), dimnames = list(1, NULL)))
+})
+
 # draws_of ----------------------------------------------------------------
 
 test_that("draws_of using with_chains works", {
@@ -111,6 +118,9 @@ test_that("unique.rvar and duplicated.rvar work", {
   unique_x_2 <- rvar(array(c(1,2, 2,3, 1,2, 3,3), dim = c(2, 2, 2)))
   expect_equal(unique(x), unique_x)
   expect_equal(unique(x, MARGIN = 2), unique_x_2)
+
+  expect_error(unique(x, MARGIN = 0), "MARGIN = 0 is invalid")
+  expect_error(unique(x, MARGIN = 3), "MARGIN = 3 is invalid")
 })
 
 # tibbles -----------------------------------------------------------------
@@ -159,13 +169,17 @@ test_that("rvars work in tibbles", {
 test_that("broadcast_array works", {
   expect_equal(broadcast_array(5, c(1,2,3,1)), array(rep(5, 6), dim = c(1,2,3,1)))
   expect_equal(
-    broadcast_array(array(1:4, c(1,4)), c(2,4)),
-    array(rep(1:4, each = 2), c(2,4))
+    broadcast_array(array(1:4, c(1,4), dimnames = list("x", letters[1:4])), c(2,4)),
+    array(rep(1:4, each = 2), c(2,4), dimnames = list(NULL, letters[1:4]))
   )
   expect_equal(
     broadcast_array(array(1:4, c(4,1)), c(4,2)),
     array(c(1:4, 1:4), c(4,2))
   )
+  expect_equal(
+    broadcast_array(array(1:2, dimnames = list(c("a","b"))), c(2,1,1,1)),
+    array(1:2, c(2,1,1,1), dimnames = list(c("a","b"), NULL, NULL, NULL))
+  )
 
   expect_error(broadcast_array(array(1:9, dim = c(3,3)), c(1,9)))
   expect_error(broadcast_array(array(1:9, dim = c(3,3)), c(9)))
